'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _waterline = require('waterline');

var _waterline2 = _interopRequireDefault(_waterline);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _async = require('async');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _lodash = require('lodash');

var _objectPath = require('object-path');

var _changeCase = require('change-case');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _clone = require('clone');

var _clone2 = _interopRequireDefault(_clone);

var log = (0, _debug2['default'])('express:waterline');

var METHODS = [
// GET
'beforeAccess', 'afterAccess',

// PUT, POST
'beforeSave', 'afterSave',

// PUT
'beforeUpdate', 'afterUpdate',

// POST
'beforeCreate', 'afterCreate',

// DELETE
'beforeDelete', 'afterDelete'];

var passThrough = function passThrough(data) {
  return _bluebird2['default'].resolve(data);
};

exports['default'] = function (config) {
  var _this = this;

  var connections = (0, _objectPath.get)(config, 'connections', {});
  var multiTenant = {};
  config.connections = Object.keys((0, _objectPath.get)(config, 'connections', {})).reduce(function (connections, key) {
    var tenants = connections[key];
    if (Array.isArray(tenants)) {
      multiTenant[key] = tenants.map(function (tenant) {
        var id = (0, _objectPath.coalesce)(tenant, ['tenantId', 'tenantid']);
        (0, _assert2['default'])(!!id, 'Field "tenantId" required for multi-tenant connections');
        id = id.toLowerCase();
        log('Creating tenant ' + key + ' ' + id);
        if (id !== 'default') connections[key + '-' + id] = tenant;else connections[key] = tenant;
        return id;
      });
      if (Array.isArray(connections[key])) delete connections[key];
    }

    return connections;
  }, connections);

  log('Initializing models');
  // Instantiate a new instance of the ORM
  var orm = new _waterline2['default']();

  var schema = _glob2['default'].sync(_path2['default'].join(config.dir, '**/*.js')).map(require).reduce(function (schema, schemata) {
    schema[schemata.identity] = schemata;
    return schema;
  }, {});

  var models = Object.keys(schema).map(getModel).map(function (model) {
    model.lifecycle = getLifecycle(model);

    model = (0, _lodash.merge)({}, config.defaults, model);

    if (!model.tableName) model.tableName = (0, _changeCase.pascalCase)(model.identity);
    return model;
  }).filter(function (model) {
    return model['public'] !== false && !!model.connection;
  });

  models.reduce(function (prev, curr) {
    var connection = curr.connection;

    var tenants = multiTenant[connection];
    if (Array.isArray(tenants)) {
      tenants.map(function (tenant) {
        var obj = (0, _clone2['default'])(curr, false);
        var suff = suffix.bind(_this, tenant);
        if (tenant !== 'default') {
          suff(obj, 'identity');
          suff(obj, 'connection');

          // Adding suffixes to foreign key attributes
          Object.keys(obj.attributes).map(function (key) {
            suff(obj, ['attributes', key, 'model']);
            suff(obj, ['attributes', key, 'collection']);
          });
        }
        log('Adding model ' + obj.identity);
        prev.push(obj);
      });
    } else {
      prev.push(curr);
    }
    return prev;
  }, models).map(function (model) {
    return _waterline2['default'].Collection.extend(model);
  }).map(orm.loadCollection.bind(orm));

  return _bluebird2['default'].promisify(orm.initialize.bind(orm))(config).then(function (models) {
    log('Waterline started');
    return models.collections;
  });

  function getModel(name) {
    var obj = schema[name];
    if (!obj) throw new Error('Schmemata for model ' + name + ' not found');

    if (obj.base) obj = (0, _lodash.merge)({}, getModel(obj.base), obj, function (a, b, key) {
      if (~METHODS.indexOf(key) && a && b) {
        if (!Array.isArray(a)) a = [a];
        if (!Array.isArray(b)) b = [b];
        return a.concat(b);
      }
    });

    return obj;
  }

  function getLifecycle(model) {
    return METHODS.reduce(function (prev, method) {
      var fn = (0, _objectPath.get)(model, ['lifecycle', method], passThrough);
      (0, _objectPath.set)(prev, method, execLifecycle.bind(model, fn));
      return prev;
    }, {});
  }

  function suffix(suffix, obj, key) {
    var val = (0, _objectPath.get)(obj, key);
    if (val) (0, _objectPath.set)(obj, key, (0, _objectPath.get)(obj, key) + '-' + suffix);
    return obj;
  }

  function execLifecycle(lifecycle) {
    var _this2 = this;

    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    log('exec lifecycle %s', lifecycle);
    var instance = args[0];

    if (!instance) instance = args[0] = {};
    if (!lifecycle) lifecycle = function () {
      return _bluebird2['default'].resolve(instance);
    };
    if (!Array.isArray(lifecycle)) lifecycle = [lifecycle];

    return _bluebird2['default'].reduce(lifecycle, function (prev, mw) {
      return _bluebird2['default'].method(mw).apply(_this2, args).then(function (ins) {
        if (typeof ins !== 'undefined') return ins;
        return instance;
      });
    }, instance);
  }
};

module.exports = exports['default'];