'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.cleanup = cleanup;
exports.coerce = coerce;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _boolean = require('boolean');

var _boolean2 = _interopRequireDefault(_boolean);

exports['default'] = function (model) {
  var _ref = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var where = _ref.where;
  var limit = _ref.limit;
  var include = _ref.include;
  var populate = _ref.populate;
  var fields = _ref.fields;
  var select = _ref.select;
  var order = _ref.order;
  var sort = _ref.sort;
  var offset = _ref.offset;

  // Aliases
  if (sort) order = sort;
  if (populate) include = populate;
  if (fields) select = fields;

  if (include && !Array.isArray(include)) {
    if (typeof include === 'object') include = Object.keys(include);
    if (typeof include === 'string') include = [include];
  }
  if (typeof select === 'string') select = [select];

  var params = undefined;
  var method = 'find';
  var isCount = !!where && where.id === 'count';

  if (select) params = { select: select };
  if (where && where.id) {
    if (isCount) {
      method = 'count';
    } else {
      method += 'One';
      params = { id: where.id };
    }

    delete where.id;
  }

  var query = model[method](params);

  if (where) query.where(cleanup(where, model));
  if (!isCount) {
    if (order) query.sort(order);
    if (include) include.map(function (incl) {
      return query.populate(incl);
    });
    if (offset) query.skip(offset);
    if (limit) query.limit(limit);
  }

  return query.then(function (data) {
    if (isCount) data = { count: data };
    return data;
  });
};

function cleanup(where, model) {
  return Object.keys(where).reduce(function (obj, prop) {
    // TODO: Probably bad practice to reference underscored properties...
    var castType = model._cast._types[prop];
    obj[prop] = castType ? coerce(where[prop], castType) : where[prop];
    return obj;
  }, {});
}

function coerce(val, type) {
  if (val === null || val === undefined) return val;
  if (Array.isArray(val)) return val.map(function (v) {
    return coerce(v, type);
  });
  if (typeof val === 'object') return Object.keys(val).reduce(function (prev, curr) {
    prev[curr] = coerce(val[curr], type);
    return prev;
  }, {});
  if (type === 'string') return val.toString();
  if (type === 'integer') return parseInt(val);
  if (type === 'double' || type === 'float') return parseFloat(val);
  if (type === 'boolean') return (0, _boolean2['default'])(val);
}